#################################################################################
# File: 			all_primes_array.tcl
# Purpose:			General math test
# Author: 			M Shaw
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 13/Jan/2018	First version
# 1.01 01/Jan/2020	Updated
#################################################################################
# Notes:
# Not part of main tests bundle
# Implements https://wiki.tcl.tk/951 with additional checksums
# all_primes_array is much faster than list based algorithm
# Tested and checked up to 1,000,000
#################################################################################

/* ////////////////////////////////////////////////////////////////////////////////
http://list.seqfan.eu/pipermail/seqfan/2009-February/000930.html
> A046731  Sum of primes < 10^n. (which is the first term of the sequences
> below:
> 0, 17, 1060, 76127, 5736396, 454396537, 37550402023, 3203324994356,
> 279209790387276, 24739512092254535, 2220822432581729238,
> 201467077743744681014, 18435588552550705911377,
> 1699246443377779418889494, 157589260710736940541561021,
> 14692398516908006398225702366, 1376110854313351899159632866552,
> 129408626276669278966252031311350, 12212914292949226570880576733896687,
> 1156251260549368082781614413945980126,
> 109778913483063648128485839045703833541,
> 10449550362130704786220283253063405651965
//////////////////////////////////////////////////////////////////////////////// */

option expression on

set count 1
set primes(0) 2			# Set global

/* C version? ////////////////////////////////////////////////////////////////////
long* a=NULL;
cont MAX_INT 100000;
primes=(__int64*) malloc(sizeof(__int64)*MAX_INT);
primes(0)=2
long count=1;

void all_primes_array(long max)
{
	long acount=0;
	for(__int64 test=3; test < max; ++ test)
	{
		long max_test=sqrt(test);
		if(count >= MAX_INT)
			break;
		for(i=0; i < acount; i++)
		{
			if(primes(i) > max_test)
			{
				primes(count)=test
				count++;
				break;
			}
			if( (test % primes(count)) == 0)
				break;
		}
	}
}
*/
#################################################################################

undef all_primes_array -nocomplain
proc all_primes_array {max} {
	for {set test 3} {$test <= $max} {++ test 2} {
		set maxTest [expr {int(sqrt($test))}]
		set acount [array size ::primes]
		for {set i 0} {$i < $acount} {++ i} {
			if {$::primes($i)  > $maxTest} {
				# puts "Storing: '$test'"
				set ::primes($::count) $test
				++ ::count
				break
			}
			if {! [expr {$test % $::primes($i)}]} {
				break
			}
		}
	}
}

undef all_primes_run_array -nocomplain
proc all_primes_run_array {} {
	puts -nonewline "Compute all primes below:  "
	flush stdout
	set max [gets stdin]
	if {$max < 2} {
		set max 2
	}
	puts "Computing primes $max"
	all_primes_array $max

	puts "Done. Writing to primes_below_${max}.txt"

	#array walk ::primes

	puts "Writing '\$::primes'"
	# mode "w" will translate \n 0x0a to \r\n 0x0d0x0a so "\r\n" -> \r\r\n 0x0d0x0d0x0a
	# puts and printf would need to know which mode a handle had been opened in
	set f [open "primes_below_${max}.txt" wb]
	set acount [array size ::primes]
	for {set i 0} {$i < $acount} {++ i} {
		puts $f "$::primes($i) " -nonewline		# FIX 0D 0D 0A terminator in mode "w", ok in mode "wb" when using puts $f $primes
	}
	close $f

	puts "Calculating the sum:"
	set sum 0
	for {set i 0} {$i < $acount} {++ i} {
		#puts "Add $x"
		+= sum $::primes($i)
	}
	puts "Sum is $sum"
	if {$max == 10} {
		assert $sum {eq $_ 17} -v -line #__LINE__			# https://codereview.stackexchange.com/questions/75640/haskell-monads-sum-of-primes
	} elseif {$max == 100} {
		assert $sum {eq $_ 1060} -v -line #__LINE__			# https://codereview.stackexchange.com/questions/75640/haskell-monads-sum-of-primes
	} elseif {$max == 1000} {								# 
		assert $sum {eq $_ 76127} -v -line #__LINE__		# https://codereview.stackexchange.com/questions/75640/haskell-monads-sum-of-primes
	} elseif {$max == 10000} {								# 
		assert $sum {eq $_ 5736396} -v -line #__LINE__		# https://answers.yahoo.com/question/index?qid=20130801125156AAIUsQ6
	} elseif {$max == 25000} {								# 
		assert $sum {eq $_ 32405717} -v -line #__LINE__	# assert (32405717 == primes_sum4 25000)
	} elseif {$max == 100000} {								# https://codereview.stackexchange.com/questions/75640/haskell-monads-sum-of-primes
		assert $sum {eq $_ 454396537} -v -line #__LINE__	# (454396537) 35610 milliseconds with output, 32020 without
	} elseif {$max == 1000000} {							#
		assert $sum {eq $_ 37550402023} -v -line #__LINE__	# Self calculated  559862 milliseconds
	}
	# 142913828922 == primes_sum4 2000000
	@ cmd /c type "primes_below_${max}.txt"
}

set start_time [timer]
all_primes_run_array

puts "Took [- [timer] $start_time] milliseconds"
newline

textcolor green
puts "* Done *"
textcolor

# EOF #